#include "hal_nrf.h"
#include "hal_clk.h"
#include <hal_uart.h>
#include "hal_rtc.h"
#include "hal_delay.h"

#include <stdio.h>

//----------------  PRINTF -----------------------------------
// Cusomization of low level stdio function. Used by for example printf().
#ifdef __ICC8051__
int putchar(int c)
#else /*presume C51 or other accepting compilator*/
char putchar(char c)
#endif
{
  hal_uart_putchar(c);
  return c;
}

// Cusomization of low level stdio function. Used by for example gets().
#ifdef __ICC8051__
//	#define getchar(x) ((int) hal_uart_getchar())
#else
//	#define getchar(x) ((char) hal_uart_getchar())
#endif


#ifdef __ICC8051__
//int getchar(void)   { return hal_uart_getchar();}
#else /*presume C51 or other accepting compilator*/
//char getchar(void) {  return hal_uart_getchar();}
#endif

// Repeated putchar to print a string
void putstring(char *s)
{
  while(*s != 0)
    putchar(*s++);
}



void uart_begin(void) {
	EA = 1;
	P0DIR = (P0DIR & ~0x08) | 0x10; // bxxx1-0xxx
	hal_uart_init(UART_BAUD_38K4);     //< must be 38.4 kbaud -- why? max for 
  while(hal_clk_get_16m_source() != HAL_CLK_XOSC16M);
}

/**
 flush all data in the buffer and close the connection
*/
void uart_end(void) {
	while(!hal_uart_tx_complete());
	REN0 = 0; // disable serial reception
	//S0CON &= ~0x;
}

/**
	wait for uart buffer to be empty
*/
void wait_uart(void) {
		while(!hal_uart_tx_complete());
}

//----------------------------------------------------


// Global variables
bool radio_busy;


uint8_t packetId;

bool got_ack_pack;
uint8_t rx_payload[32];
uint8_t len;

#ifdef ROLE_PTX																			// use different pins for signalling status, depending on PTX or PRX role.
	#define led_pin P02
#else																								// it's ROLE_PRX
	#define led_pin P12
#endif



uint8_t xdata payload[32];

void initialize(void) {
  int i;
	//uint8_t payload[3];
	for (i=0; i < 32; i++) payload[i]=i;
	
}

void rf_init(void) {
	  // Enable the radio clock
  RFCKEN = 1;

  // Enable RF interrupt
  RF = 1;  

  // Enable global interrupt
  EA = 1;

	hal_nrf_set_datarate(HAL_NRF_250KBPS);
	hal_nrf_set_auto_retr(3U, 1500);

  // Enable dynamic payload on all pipes
  hal_nrf_setup_dynamic_payload(0xFF);
  //Enable Dynamic payload
  hal_nrf_enable_dynamic_payload(1);
  // Enable ACK with payload
  hal_nrf_enable_ack_payload(1);
  hal_nrf_enable_dynamic_ack(1);				// need this to be able to get & read ack payload sent by prx.


  // Power up radio
  hal_nrf_set_power_mode(HAL_NRF_PWR_UP);
}

void send(void) {
		EA = 1;
    // Write payload to radio TX FIFO
    hal_nrf_write_tx_payload(payload, 32);
    // Toggle radio CE signal to start transmission 
    CE_PULSE();

    radio_busy = true;

    // Wait for radio operation to finish
    while(radio_busy);
		printf("Sent.");

		if (got_ack_pack) {
			printf(" Ack pack:length:%d, ",(unsigned int) len);
			printf("Payload: %02X, %02X, %02X\n", (int) rx_payload[0], (int) rx_payload[1], (int) rx_payload[2]); // for testing.
			rx_payload[0]=rx_payload[1]=rx_payload[2];	// just for testing.
			got_ack_pack = false;
		}
		wait_uart();
}

void mcu_init(void) {
	P0DIR |= 0x13;		// bxxx1-xx11. RXD = P04, XOSC32K inputs= P01, P02. Set bits to 1 for input.
	P0DIR &= 0xF3;  	// ~0x0C, or 0xF3: bxxxx-00xx. TXD = P03, LED = P02. Set bits to 0 for output.
	P1DIR |= 0x02; 		// bxxxx-x1xx. ADC AIN0 = P12. Set bit to 1 for input.

	OPMCON = 0x00; 		// open latch
}

void disconnectGPIO()
{
  uint8_t i;
  for (i = 0; i < 8; i ++)
  {
    P0CON = 0x70 | i;
    P1CON = 0x70 | i;
    //P2CON = 0x70 | i;
    //P3CON = 0x70 | i;
  }
}

void set_timer(void) {
	hal_rtc_start(false);
	hal_clklf_set_source(HAL_CLKLF_RCOSC32K); 
	hal_rtc_set_compare_mode(HAL_RTC_COMPARE_MODE_0);
	hal_rtc_set_compare_value(0x7FFF);
	hal_rtc_start(true); 
}

void main()
{
	int test = 1;															// various test cases for sending/sleeping.

	mcu_init(); 															// do this every time we go wakey wakey
	if (PWRDWN & 0x40) {											// do repetitive wakey stuff
		//hal_rtc_start(false);
		rf_init();
		
		uart_begin();
		EA = 1;
		printf("\nwoke up:\n");
		wait_uart();
		
		switch (test) {
			case (1): // works, but no sleep:
			default:
				while(1) { send();}
				break;
			case (2):	// sleeps, sends packet, gets ACK that packet recd, BUT rx payload packet isn't updated after the first time. PRX 
				send();
				uart_end();
				break;
			case (3):	// sleeps, sends TWO packets.
				send();
				send();
				uart_end();
				break;
		}

	} else {
		initialize();

		// set RTC stuff
		set_timer();
	}
	disconnectGPIO();
	OPMCON |= 0x02; 			// bxxxx-xx0x is latch open, bxxxx-xx1x is latch closed.
	PWRDWN = 0x03; 				// Memory retention mode timer on, system reset on wakeup
	//PWRDWN = 0x04; 			// register retention mode
	while(1);							// wait to go to sleep	
} 

// Radio interrupt
NRF_ISR()
{
  uint8_t irq_flags;


  // Read and clear IRQ flags from radio
  irq_flags = hal_nrf_get_clear_irq_flags(); 

  switch(irq_flags)
  {
    // Transmission success
    case (1 << HAL_NRF_TX_DS):
      radio_busy = false;
      // Data has been sent
      break;
    // Transmission failed (maximum re-transmits)
    case (1 << HAL_NRF_MAX_RT):
      // When a MAX_RT interrupt occurs the TX payload will not be removed from the TX FIFO. 
      // If the packet is to be discarded this must be done manually by flushing the TX FIFO.
      // Alternatively, CE_PULSE() can be called re-starting transmission of the payload.
      // (Will only be possible after the radio irq flags are cleared) 
      hal_nrf_flush_tx();
      radio_busy = false;
      break;
    default:
    radio_busy = false;
    break;
  }
  if(!hal_nrf_rx_fifo_empty())
  {
    
    // Read out rx_payload
    len = hal_nrf_read_rx_payload(rx_payload);
    // Output byte 1 on P0    
    //P0 = ~rx_payload[1];
		got_ack_pack = true;

  }
}